#########################################################################
# cmmucl.tcl - Console version of mmucl.
#
# Copyright (C) 1997-1998 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Make the package mechanism look in the right place
set auto_path [linsert $auto_path 0 $env(MMUCL_DIR)]

package require mmucl

proc term_save_pos {} {
    puts -nonewline \0337
}

proc term_restore_pos {} {
    puts -nonewline \0338
}

proc term_goto_rowcol {row col} {
    puts -nonewline \033\[${row}\;${col}f
}

proc term_erase_screen {} {
    puts -nonewline \033\[2J
}

proc term_erase_toeol {} {
    puts -nonewline \033\[K
}

proc term_reset {} {
    puts -nonewline \033c
}

proc term_scroll_region {top bottom} {
    puts -nonewline \033\[${top}\;${bottom}r
}

namespace eval mmucl {
}

array set mmucl {
    echo            1
    echo_attribs    {yellow}
    hist_min_length 3
}

set mmucl::Priv(newline) 1

proc mmucl::display {str} {
    variable Priv

    term_save_pos
    term_goto_rowcol $Priv(displayline) 0
    puts -nonewline $str
    term_restore_pos
    flush stdout
    set Priv(newline) [string match *\n $str]
}

# Make split screen
proc mmucl::MCsplit {} {
    global env
    variable Priv

    set rows 24.
    if {[info exists env(LINES)]} {
	set rows $env(LINES)
    } else {
	catch {set rows [lindex [exec stty size] 0]}
    }
    
    term_reset
    term_erase_screen
    term_scroll_region 1 [expr {$rows-2}]
    term_goto_rowcol [expr {$rows-1}] 0
    puts " ---- "
    set Priv(displayline) [expr {$rows-2}]
}

proc mmucl::MChelp {{subject contents}} {
    global mmucl env
    
    set file [file join $env(MMUCL_DIR) html $subject.html]
    if {![file exists $file]} {
 	# should go to some kind of error page
 	set file [file join $env(MMUCL_DIR) html contents.html]
    }

    display [exec lynx -dump $file]
}

proc mmucl::meta_parse {} {
    global mmucl
    
    if {[gets stdin input] < 0} {
	MCexit
    }
    
    term_erase_toeol
    if {[string compare $mmucl(echo) 0]} {
	MCecho $input [split $mmucl(echo_attribs)]
    }

    MCparse $input

    if {[string length $input] >= $mmucl(hist_min_length)} {
	history add $input
    }
}

rename exit _exit
proc exit {} {
    term_reset
    _exit
}

mmucl::MCsplit

# Accept input from the console.
fileevent stdin readable {mmucl::meta_parse}
fconfigure stdin -blocking 0 -buffering line

mmucl::init $argv

# Override default echo to fix overwrite prob
proc mmucl::MCecho {str {attribs {}}} {
    variable Priv

    if {!$Priv(newline)} {append msg \n}
    append msg [toansi $str $attribs]\n
    display $msg
}

# Enter the event loop.
vwait forever