package provide form 0.11

#########################################################################
# form.tcl - A pseudo-widget for making dialog boxes to edit arrays.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################



proc form {args} {
    if {[llength $args]<5} {
	return -code error "wrong # args: form w -array name -labels list\
-addCallback proc -deleteCallback proc"
    }

    set w [lindex $args 0]
    if {[catch {frame $w -class Form} error]} {
	return -code error $error
    }
    
    foreach {opt value} [lrange $args 1 end] {
	switch -glob -- $opt {
	    -arr* {
		set arrayName $value
	    }
	    -lab* {
		set labels $value
	    }
	    -add* {
		set addCmd $value
	    }
	    -del* {
		set delCmd $value
	    }
	    default {
		return -code error "invalid option $opt"
	    }
	}
    }

    frame $w.sel
    pack [listbox $w.sel.list -yscroll [list $w.sel.yscroll set]] -side left
    pack [scrollbar $w.sel.yscroll -command [list $w.sel.list yview]] \
	      -side left -fill y

    frame $w.edit
    frame $w.edit.buts
    pack [button $w.edit.buts.add -text Add -command [list form::curAdd $w]]\
	-side left
    pack [button $w.edit.buts.delete -text Delete \
	      -command [list form::curDelete $w]] -side left
    frame $w.edit.f
    set count 0
    foreach label $labels {
	grid [label $w.edit.f.l$count -text $label] -sticky w -column 0
	grid [entry $w.edit.f.e$count] -row $count -sticky ew -column 1
	incr count
    }
    
    pack $w.edit.f $w.edit.buts -side top
    pack $w.sel $w.edit -side left
   
    bind $w.edit.f.e[expr {$count-1}] <Key-Return> [list form::curAdd $w]
    bind $w.sel.list <ButtonRelease-1> [list form::curUpdate $w $arrayName]

    interp alias {} $w {} form::formCmd $w $arrayName $addCmd $delCmd
    form::fupdate $w $arrayName
    return $w
}

namespace eval form {}

proc form::formCmd {w arrayName addCmd delCmd op args} {
    switch -glob -- $op {
	listbox {
	    return [eval $w.sel.list $args]
	} entry   {
	    set i [lindex $args 0]
	    return [eval $w.edit.f.e$i [lrange $args 1 end]]
	} label {
	    set i [lindex $args 0]
	    return [eval $w.edit.f.l$i [lrange $args 1 end]]
	} set {
	    return [eval formCmd_set $w $arrayName $addCmd $args]
	} delete {
	   eval formCmd_delete $w $arrayName $delCmd $args
	} default {
	    return -code error "bad option \"$op\": Must be listbox, entry, label, set, delete"
	}
    }
}

proc form::formCmd_set {w arrayName addCmd args} {
    upvar #0 $arrayName array

    set i [lindex $args 0]
    if {[llength $args]>1} {
	set ret [eval $addCmd [concat [list $i] [lindex $args 1]]]
    } else {
	if {$i==""} {
	    set ret [array names array]
	} else {
	    set ret [set array($i)]
	}
    }
    fupdate $w $arrayName
    return $ret
}

proc form::formCmd_delete {w arrayName delCmd args} {
    upvar #0 $arrayName array

    set i [lindex $args 0]
    $delCmd $i
    catch {unset array($i)}
    fupdate $w $arrayName
}

proc form::curUpdate {w arrayName} {
    upvar #0 $arrayName array
    
    set i [curIndex $w]
    if {$i!=""} {
	set list [concat [list $i] $array($i)]
    } else {
	set list {}
    }
    fput $w $list
}

proc form::curAdd {w} {
    set list [fget $w]
    set i [lindex $list 0]
    
    if {$i!=""} {
	$w set  $i [lrange $list 1 end]
	fput $w {}
    }
}

proc form::curDelete {w} {
    $w delete [curIndex $w]
    fput $w {}
}

proc form::curIndex {w} {
    set i [$w.sel.list curselection]
    if {$i!=""} {
	return [$w.sel.list get $i]
    }
}

proc form::fupdate {w arrayName} {
    upvar #0 $arrayName array
 
    $w.sel.list delete 0 end
    foreach i [lsort [array names array]] {
        $w.sel.list insert end $i
    }
}

proc form::fget {w} {
    set num_entries [expr {[llength [winfo children $w.edit.f]]/2}]
    for {set i 0} {$i<$num_entries} {incr i} {
	lappend list [$w.edit.f.e$i get]
    }
    return $list
}

proc form::fput {w list} {
    set num_entries [expr {[llength [winfo children $w.edit.f]]/2}]
    for {set i 0} {$i<$num_entries} {incr i} {
	set str [lindex $list $i]
 	$w.edit.f.e$i delete 0 end
 	$w.edit.f.e$i insert end $str
    }
}
