package provide htmlHelp 0.21
package require textHtml

##########################################################################
# htmlHelp.tcl - A simple html browser for a help system.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##########################################################################

# Don't use own namespace because of difficulty of defining
# callbacks needed for textHtml package.
namespace eval textHtml {
    variable htmlHelp

    array set htmlHelp {
	history {}
	histloc 0
	file    {}
	status  {}
    }
}

proc tk_htmlHelp {file} {

    set w .htmlHelp

    if {[winfo exists $w]} {
	raise $w
	return
    }

    toplevel $w -class HtmlHelp

    frame $w.toolbar
    button $w.toolbar.back -text Back -command {textHtml::history_move 1}
    button $w.toolbar.forward -text Forward \
	-command {textHtml::history_move -1}
    button $w.toolbar.exit -text Exit \
	-command {
	    textHtml::reset_win .htmlHelp.display
	    textHtml::set_state .htmlHelp -stop 1
	    destroy .htmlHelp
	}
    pack $w.toolbar.back $w.toolbar.forward -side left    
    pack $w.toolbar.exit -side right

    text $w.display -yscrollcommand [list $w.vscroll set] -wrap word
    scrollbar $w.vscroll -orient vertical -command [list $w.display yview]

    label $w.status -textvariable textHtml::htmlHelp(status) -foreground blue

    grid $w.toolbar -sticky ew
    grid $w.display -row 1 -column 0 -sticky news
    grid $w.vscroll -row 1 -column 1 -sticky ns
    grid $w.status -row 2 -column 0 -sticky ew
    grid rowconfigure $w 1 -weight 1
    grid columnconfigure $w 0 -weight 1

    bind $w <End> [list $w.display yview end]
    bind $w <Home> [list $w.display yview 0.0]
    bind $w <Next> [list $w.display yview scroll 1 page]
    bind $w <Prior> [list $w.display yview scroll -1 page]
    bind $w <Key-Up> [list $w.display yview scroll -1 units]
    bind $w <Key-Down> [list $w.display yview scroll 1 units]

    textHtml::init_win $w.display


    # Hack to set right font size for display
    set size [lindex [$w.display cget -font] 1]
    textHtml::set_state $w.display -size [expr {$size-12}]

     array set textHtml::htmlHelp {
	 history {}
	 histloc 0
	 file    {}
	 status  {}
     }
    
    textHtml::display $file
    
    focus $w.display
}

proc textHtml::link_callback {win href} { 
    variable htmlHelp
    
    if {[string match \#* $href]} {
        status "Going to [string range $href 1 end]"
        display $href
        return
     }

    if {[regexp {([^:]+):(.+)} $href x type href]} {
        if {$type!="file"} {
            status "$type not supported"
            return
        }
    }

    # Change href to file and make it absolute
    if {[file pathtype $href]=="relative"} {
        set htmlHelp(file) [file join [file dirname $htmlHelp(file)] $href]
    } else {
        set htmlHelp(file) $href
    }
    display $htmlHelp(file)
}

proc textHtml::link_setup {win href} {
     regsub -all {%} $href {%%} href
     
     $win tag bind L:$href <Enter> \
	 "$win configure -cursor hand2\;set textHtml::htmlHelp(status) {$href}"
     $win tag bind L:$href <Leave> \
	 "$win configure -cursor {}\;set textHtml::htmlHelp(status) {}"
     $win tag bind L:$href <1> [list $win configure -cursor {}]
 }


proc textHtml::display {file {add_hist 1}} {
    variable htmlHelp

    set fragment ""
    regexp {([^#]*)#(.+)} $file dummy file fragment
     if {$file == "" && $fragment != ""} {
	goto .htmlHelp.display $fragment
        return
    }

    .htmlHelp.display configure -state normal
    reset_win .htmlHelp.display
    status "loading $file"
    update idletasks
	      
    parse_html [get_html $file] {render .htmlHelp.display}
    set htmlHelp(file) $file
    
    set_state .htmlHelp.display -stop 1
    status ""
    
    .htmlHelp.display configure -state disabled

     if {$add_hist} {
         set htmlHelp(history) \
	     [linsert $htmlHelp(history) $htmlHelp(histloc) $file]
   }
}

proc textHtml::get_html {file} {

    if {[catch {set fd [open $file]} error]} {
        set html "<H1>Error: $error</H1>"
        status $error
    } else {
        set html [read $fd]
        close $fd
    }
    return $html
}

proc textHtml::status {msg} {
    variable htmlHelp
    
    set msg [string range $msg 0 [.htmlHelp.display cget -width]]
    set htmlHelp(status) $msg
}

proc textHtml::history_move {dir} {
    variable htmlHelp

    incr htmlHelp(histloc) $dir
    set file [lindex $htmlHelp(history) $htmlHelp(histloc)]

    if {$file==""} {
         incr htmlHelp(histloc) -$dir
    } else {
        display $file 0
    }
}
